!> @file user_init.f90
!------------------------------------------------------------------------------!
! This file is part of PALM.
!
! PALM is free software: you can redistribute it and/or modify it under the
! terms of the GNU General Public License as published by the Free Software
! Foundation, either version 3 of the License, or (at your option) any later
! version.
!
! PALM is distributed in the hope that it will be useful, but WITHOUT ANY
! WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
! A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
!
! You should have received a copy of the GNU General Public License along with
! PALM. If not, see <http://www.gnu.org/licenses/>.
!
! Copyright 1997-2017 Leibniz Universitaet Hannover
!------------------------------------------------------------------------------!
!
! Current revisions:
! -----------------
! 
! 
! Former revisions:
! -----------------
! $Id: user_init.f90 2101 2017-01-05 16:42:31Z suehring $
!
! 2000 2016-08-20 18:09:15Z knoop
! Forced header and separation lines into 80 columns
! 
! 1799 2016-04-05 08:35:55Z gronemeier
! Bugfix: added dots_num to variable list
!
! 1783 2016-03-06 18:36:17Z raasch
! netcdf module name changed + related changes
!
! 1682 2015-10-07 23:56:08Z knoop
! Code annotations made doxygen readable 
! 
! 1353 2014-04-08 15:21:23Z heinze
! REAL constants provided with KIND-attribute 
!
! 1320 2014-03-20 08:40:49Z raasch
! revision history before 2012 removed,
! comment fields (!:) to be used for variable explanations added to
! all variable declaration statements 
!
! 1036 2012-10-22 13:43:42Z raasch
! code put under GPL (PALM 3.9)
!
! 211 2008-11-11 04:46:24Z raasch
! Former file user_interface.f90 split into one file per subroutine
!
! Description:
! ------------
!> Execution of user-defined initializing actions
!------------------------------------------------------------------------------!
 SUBROUTINE user_init
 

    USE arrays_3d,                                                             &
        ONLY:  zu

    USE control_parameters
    
    USE grid_variables,                                                        &
        ONLY:  dx, dy

    USE indices
    
    USE kinds
    
    USE netcdf_interface,                                                      &
        ONLY: dots_label, dots_unit, dots_num

    USE pegrid
    
    USE surface_mod,                                                           &
        ONLY:  surf_def_h

    USE user

    IMPLICIT NONE

    CHARACTER (LEN=20) ::  field_char, dummy_char   !< 
    CHARACTER (LEN=4)  ::  residencefile = 'RES_'
    CHARACTER (LEN=7)  ::  char_myid = ''

    CHARACTER (LEN=13)  :: pos_vms_char !< string for ts variable names

    INTEGER(IWP) :: i         !< loop index
    INTEGER(IWP) :: j         !< loop index
    INTEGER(IWP) :: k         !< loop index
    INTEGER(IWP) :: m         !< loop index

    ALLOCATE( usss_res(nzb:nzt+1,nysg:nyng,nxlg:nxrg) )
    ALLOCATE( vsss_res(nzb:nzt+1,nysg:nyng,nxlg:nxrg) )
    ALLOCATE( wsss_res(nzb:nzt+1,nysg:nyng,nxlg:nxrg) )
    ALLOCATE( dsdt(nzb:nzt+1,nysg:nyng,nxlg:nxrg)     )
    ALLOCATE( div_res(nzb:nzt+1,nysg:nyng,nxlg:nxrg)  )

    ALLOCATE( usss_sgs(nzb:nzt+1,nysg:nyng,nxlg:nxrg) )
    ALLOCATE( vsss_sgs(nzb:nzt+1,nysg:nyng,nxlg:nxrg) )
    ALLOCATE( wsss_sgs(nzb:nzt+1,nysg:nyng,nxlg:nxrg) )

    ALLOCATE( d_usss_res(nzb:nzt+1,nysg:nyng,nxlg:nxrg) )
    ALLOCATE( d_vsss_res(nzb:nzt+1,nysg:nyng,nxlg:nxrg) )
    ALLOCATE( d_wsss_res(nzb:nzt+1,nysg:nyng,nxlg:nxrg) )

    ALLOCATE( d_usss_sgs(nzb:nzt+1,nysg:nyng,nxlg:nxrg) )
    ALLOCATE( d_vsss_sgs(nzb:nzt+1,nysg:nyng,nxlg:nxrg) )
    ALLOCATE( d_wsss_sgs(nzb:nzt+1,nysg:nyng,nxlg:nxrg) )

    ALLOCATE( uu(nzb:nzt+1,nysg:nyng,nxlg:nxrg) )
    ALLOCATE( vv(nzb:nzt+1,nysg:nyng,nxlg:nxrg) )
    ALLOCATE( ww(nzb:nzt+1,nysg:nyng,nxlg:nxrg) )

    ALLOCATE( u_plus(nzb:nzt+1,nysg:nyng,nxlg:nxrg) )
    ALLOCATE( v_plus(nzb:nzt+1,nysg:nyng,nxlg:nxrg) )
    ALLOCATE( w_plus(nzb:nzt+1,nysg:nyng,nxlg:nxrg) )
    ALLOCATE( u_minus(nzb:nzt+1,nysg:nyng,nxlg:nxrg) )
    ALLOCATE( v_minus(nzb:nzt+1,nysg:nyng,nxlg:nxrg) )
    ALLOCATE( w_minus(nzb:nzt+1,nysg:nyng,nxlg:nxrg) )

    ALLOCATE( ff(nzb:nzt+1,nysg:nyng,nxlg:nxrg) )
    ALLOCATE( ff_h(nzb:nzt+1,nysg:nyng,nxlg:nxrg) )
    ALLOCATE( ti(nzb:nzt+1,nysg:nyng,nxlg:nxrg) )

    ALLOCATE( s_previous(nzb:nzt+1,nysg:nyng,nxlg:nxrg) )

    usss_res = 0.0_wp
    vsss_res = 0.0_wp
    wsss_res = 0.0_wp
    dsdt     = 0.0_wp

    usss_sgs = 0.0_wp
    vsss_sgs = 0.0_wp
    wsss_sgs = 0.0_wp

    d_usss_res = 0.0_wp
    d_vsss_res = 0.0_wp
    d_wsss_res = 0.0_wp

    d_usss_sgs = 0.0_wp
    d_vsss_sgs = 0.0_wp
    d_wsss_sgs = 0.0_wp

    uu = 0.0_wp
    vv = 0.0_wp
    ww = 0.0_wp

    u_plus = 0.0_wp
    v_plus = 0.0_wp
    w_plus = 0.0_wp
    u_minus = 0.0_wp
    v_minus = 0.0_wp
    w_minus = 0.0_wp

    ff   = 0.0_wp
    ff_h = 0.0_wp

    ti = 0.0_wp

    s_previous = 0.0_wp

!
!-- Initialize surface scalarflux with scalar source field from NetCDF
!-- topography file
    IF ( passive_scalar ) THEN
       IF ( ALLOCATED( s_source ) )  THEN
          DO  m = 1, surf_def_h(0)%ns
             i = surf_def_h(0)%i(m)
             j = surf_def_h(0)%j(m)
             k = surf_def_h(0)%k(m)
             surf_def_h(0)%ssws(m) = s_source(j,i)
          ENDDO
       ENDIF
    ENDIF

!
!-- Initialize virtual measurement stations (vms, timeseries)
!-- First, get total number of vms and convert metre into grid points
    n_vms = 1
    DO WHILE ( pos_vms_x(n_vms) /= -9999  .AND.  n_vms <= 100 )
       ! set vms ontop of topography
       pos_vms(n_vms,1) = INT( pos_vms_x(n_vms) / dx - 0.5 ) + 1
       pos_vms(n_vms,2) = INT( pos_vms_y(n_vms) / dy - 0.5 ) + 1
       DO  k = nzb, nzt
          IF  ( zu(k) > pos_vms_z(n_vms) )  EXIT
          pos_vms(n_vms,3) = k
       ENDDO
       n_vms = n_vms + 1
    ENDDO
    n_vms = n_vms - 1
!
!-- Define timeseries name and unit of vms
    DO  m = 1, n_vms
       WRITE ( pos_vms_char, '(A,I4.4,A,I4.4)' ) '_', pos_vms(m,1), '_', pos_vms(m,2)
       dots_label(dots_num+m) = 'u'//TRIM(pos_vms_char)
       dots_unit(dots_num+m)  = 'm/s'
       dots_label(dots_num+n_vms+m) = 'v'//TRIM(pos_vms_char)
       dots_unit(dots_num+n_vms+m)  = 'm/s'
       dots_label(dots_num+2*n_vms+m) = 's'//TRIM(pos_vms_char)
       dots_unit(dots_num+2*n_vms+m)  = 'kg/kg'
    ENDDO

    dots_num_palm = dots_num
    dots_num = dots_num + (3*n_vms)
!
!-- Open ASCII-file to output particle lifetime
    IF ( residence )  THEN
       WRITE (char_myid,'(''_'',I6.6)')  myid
       
       IF ( TRIM(initializing_actions) /= 'read_restart_data' )  THEN
          OPEN( 220, FILE= TRIM( residencefile ) // char_myid, FORM='FORMATTED' )
       ELSEIF ( TRIM(initializing_actions) == 'read_restart_data' )  THEN
          IF ( simulated_time >= 100.0    .AND. simulated_time <= 999.9    )   &
             WRITE(dummy_char,'(I3)') INT(simulated_time)
          IF ( simulated_time >= 1000.0   .AND. simulated_time <= 9999.9   )   &
             WRITE(dummy_char,'(I4)') INT(simulated_time)
          IF ( simulated_time >= 10000.0  .AND. simulated_time <= 99999.9  )   &
             WRITE(dummy_char,'(I5)') INT(simulated_time)
       
          OPEN( 220, FILE= TRIM( residencefile ) // char_myid // '_ST_' // TRIM(dummy_char), FORM='FORMATTED' )
       ENDIF
    ENDIF
    
!
!-- Ghost-point exchange for flag array. Is already allocated in NetCDF input
    IF ( ALLOCATED( analysis_flag ) )                                          &
       CALL exchange_horiz_int( analysis_flag, nys, nyn, nxl, nxr, nzt, nbgp ) 


 END SUBROUTINE user_init
